#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2025, Christoph Schober (christoph.schober@th-deg.de)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r"""
module: gitlab_invitation
short_description: Creates/updates/deletes/ GitLab User Invitations
description:
notes:
author:
  - Christoph Schober (schober-ch)
requirements:
  - python-gitlab python module
extends_documentation_fragment:
  - community.general.auth_basic
  - community.general.gitlab
  - community.general.attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  gitlab_group:
    description:
      - The C(full_path) of the GitLab group the member is added to/removed from.
      - Mutually exclusive with O(gitlab_project).
    required: true
    type: str
  gitlab_project:
    description:
      - The C(full_path) of the GitLab project the member is added to/removed from.
      - Mutually exclusive with O(gitlab_group).
    required: true
    type: str
  email:
    description:
      - A email adress or a list of email adresses to send an invitation
    type: list
    elements: str
  access_level:
    description:
      - The access level for the user.
      - Required if O(state=present), user state is set to present.
      - Mutually exclusive with O(gitlab_users_access).
    type: str
    choices: ['guest', 'reporter', 'developer', 'maintainer', 'owner']
  state:
    description:
      - State of the member in the group.
      - On V(present), it creates an invitation to a GitLab group/project.
      - On V(absent), it deletes an invitation to a GitLab group/project.
    choices: ['present', 'absent']
    default: 'present'
    type: str
"""

EXAMPLES = r"""
- name: Invite a user to a GitLab Group
  community.general.gitlab_invitation:
    api_url: 'https://gitlab.example.com'
    api_token: 'Your-Private-Token'
    gitlab_group: groupname
    email: email@email
    access_level: developer
    state: present
- name: Invite a user to a GitLab Project
  community.general.gitlab_invitation:
    api_url: 'https://gitlab.example.com'
    api_token: 'Your-Private-Token'
    gitlab_project: projectname
    email: email@email
    access_level: developer
    state: present
"""

RETURN = r""" # """

from ansible.module_utils.api import basic_auth_argument_spec
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils.gitlab import (
    auth_argument_spec, gitlab_authentication, gitlab, list_all_kwargs
)


class GitLabGroupOrProject(object):
    def __init__(self, module, gl):
        self._module = module
        self._gitlab = gl
        self.group_or_project = None

    # get group id if group exists
    def get_group(self, gitlab_group):
        return next(
            (
                g for g in self._gitlab.groups.list(search=gitlab_group, **list_all_kwargs)
                if g.full_path == gitlab_group
            ),
            None
        )

    def get_project(self, project_name):
        try:
            project_exists = self._gitlab.projects.get(project_name)
            return project_exists
        except gitlab.exceptions.GitlabGetError as e:
            project_exists = self._gitlab.projects.list(search=project_name, all=False)
            if project_exists:
                return project_exists[0]

    def get_invites(self):
        self.invites = self.group_or_project.invitations.list(all=True)

def main():
    argument_spec = basic_auth_argument_spec()
    argument_spec.update(auth_argument_spec())
    argument_spec.update(dict(
        gitlab_group=dict(type='str'),
        gitlab_project=dict(type='str'),
        email=dict(type='list', elements='str'),
        state=dict(type='str', default='present', choices=['present', 'absent']),
        access_level=dict(type='str', choices=['guest', 'reporter', 'developer', 'maintainer', 'owner']),
    ))

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ['api_username', 'api_token'],
            ['gitlab_user', 'gitlab_users_access'],
            ['access_level', 'gitlab_users_access'],
            ['api_username', 'api_oauth_token'],
            ['api_username', 'api_job_token'],
            ['api_token', 'api_oauth_token'],
            ['api_token', 'api_job_token'],
        ],
        required_together=[
            ['api_username', 'api_password'],
            # ['gitlab_user', 'access_level'],
        ],
        required_one_of=[
            ['api_username', 'api_token', 'api_oauth_token', 'api_job_token'],
            ['gitlab_group', 'gitlab_project'],
        ],
        required_if=[
            ['state', 'present', ['access_level'], True],
        ],
        supports_check_mode=True,
    )

    # check prerequisites and connect to gitlab server
    gl = gitlab_authentication(module)

    access_level_int = {
        'guest': gitlab.const.GUEST_ACCESS,
        'reporter': gitlab.const.REPORTER_ACCESS,
        'developer': gitlab.const.DEVELOPER_ACCESS,
        'maintainer': gitlab.const.MAINTAINER_ACCESS,
        'owner': gitlab.const.OWNER_ACCESS,
    }

    gitlab_group = module.params.get('gitlab_group', None)
    gitlab_project = module.params.get('gitlab_project', None)
    state = module.params['state']
    access_level = module.params['access_level']

    group_or_project = GitLabGroupOrProject(module, gl)

    if gitlab_group:
        group = group_or_project.get_group(gitlab_group)
        group_or_project.group_or_project = group
        # group doesn't exist
        if not group:
            module.fail_json(msg="group '%s' not found." % gitlab_group)
    if gitlab_project:
        project = group_or_project.get_project(gitlab_project)
        group_or_project.group_or_project = project

        if not project:
            module.fail_json(msg="project '%s' not found." % gitlab_project)

    group_or_project.get_invites()
    
    gitlab_invites = []
    emails = module.params['email']
    for email in emails:
        # TODO implement present and absent states in correct way
        # if email not in [invite.email for invite in group_or_project.invites]:
        gitlab_invites.append({'email': email, 'access_level': access_level_int[access_level] if access_level else None})

    changed = False
    error = False
    changed_invites = []
    changed_data = []
    existing_invite_emails = [invite.invite_email for invite in group_or_project.invites]
    for invite in gitlab_invites:
         
        if state == "present":
            if invite["email"] in existing_invite_emails:
                existing_invite = [inv for inv in group_or_project.invites if inv.invite_email == invite["email"]][0]
                print(existing_invite)
                if existing_invite.access_level != invite["access_level"]:
                    # existing_invite.access_level = invite["access_level"]
                    invitation = group_or_project.group_or_project.invitations.update(invite["email"], {"access_level": invite["access_level"]})
                    changed_invites.append("Successfully updated invitation for  '%s'." % invite["email"])
                    changed_data.append({'email': invite["email"], 'result': 'CHANGED',
                                                        'msg': "Successfully updated pending invite for '%s'" % invite["email"]})
                    changed = True
            else:
                invitation = group_or_project.group_or_project.invitations.create(invite)
                changed_invites.append("Successfully invited '%s'." % invite["email"])
                changed_data.append({'email': invite["email"], 'result': 'CHANGED',
                                                        'msg': "Successfully invited user with email adress '%s'" % invite["email"]})
                changed = True
        if state == "absent":
            if invite["email"] in existing_invite_emails:
                group_or_project.group_or_project.invitations.delete(invite["email"])
                changed_invites.append("Removed invite for '%s'." % invite["email"])
                changed_data.append({'email': invite["email"], 'result': 'CHANGED',
                                                        'msg': "Successfully deleted pending invite for '%s'" % invite["email"]})
                changed = True

    module.exit_json(changed=changed, msg='Successfully set invites', result="\n".join(changed_invites), result_data=changed_data)


if __name__ == '__main__':
    main()
